@@ -1,5 +1,6 @@
 from .homs import *
-from .ops import *
+from .loss_functions import *
+from .misc import *
 
 
 __all__ = [_ for _ in dir() if not _.startswith("_")]
\ No newline at end of file
